Add homepage link.
[paraslash.git] / signal.c
index 68db5df7d95cf52e6cd179ee3f4f59a02a204666..32d6ab6624e5f493ac393b630a603c0968f11497 100644 (file)
--- a/signal.c
+++ b/signal.c
@@ -1,17 +1,15 @@
-/*
- * Copyright (C) 2004-2014 Andre Noll <maan@tuebingen.mpg.de>
- *
- * Licensed under the GPL v2. For licencing details see COPYING.
- */
+/* Copyright (C) 2004 Andre Noll <maan@tuebingen.mpg.de>, see file COPYING. */
 /** \file signal.c Signal handling functions. */
 
 #include <signal.h>
 #include <sys/types.h>
+#include <regex.h>
 
 #include "para.h"
 #include "error.h"
 #include "fd.h"
 #include "list.h"
+#include "string.h"
 #include "sched.h"
 #include "signal.h"
 
@@ -23,7 +21,7 @@ static int signal_pipe[2];
  * This function creates a pipe, the signal pipe, to deliver pending
  * signals to the application (Bernstein's trick). It should be called
  * during the application's startup part, followed by subsequent calls
- * to para_install_sighandler() for each signal that should be caught.
+ * to \ref para_install_sighandler() for each signal that should be caught.
  *
  * A generic signal handler is used for all signals simultaneously. When a
  * signal arrives, the signal handler writes the number of the signal received
@@ -31,13 +29,15 @@ static int signal_pipe[2];
  * by checking if the file descriptor of the other end of the signal pipe is
  * ready for reading, see select(2).
  *
* \return This function either succeeds or calls exit(2) to terminate the
* current process. On success, the file descriptor of the read end of the
* signal pipe is returned.
  */
-int para_signal_init(void)
+struct signal_task *signal_init_or_die(void)
 {
+       struct signal_task *st;
        int ret;
+
+       PARA_NOTICE_LOG("setting up signal handling\n");
        if (pipe(signal_pipe) < 0) {
                ret = -ERRNO_TO_PARA_ERROR(errno);
                goto err_out;
@@ -48,7 +48,9 @@ int para_signal_init(void)
        ret = mark_fd_nonblocking(signal_pipe[1]);
        if (ret < 0)
                goto err_out;
-       return signal_pipe[0];
+       st = para_calloc(sizeof(*st));
+       st->fd = signal_pipe[0];
+       return st;
 err_out:
        PARA_EMERG_LOG("%s\n", para_strerror(-ret));
        exit(EXIT_FAILURE);
@@ -223,9 +225,12 @@ int para_next_signal(fd_set *rfds)
 }
 
 /**
- * Close the write end of the signal pipe.
+ * Close the write end of the signal pipe, deallocate resources.
+ *
+ * \param st The pointer obtained earlier from signal_init_or_die().
  */
-void para_signal_shutdown(void)
+void signal_shutdown(struct signal_task *st)
 {
        close(signal_pipe[1]);
+       free(st);
 }
@@ -31,13 +29,15 @@ static int signal_pipe[2];
  * ready for reading, see select(2).
  *
- * \return This function either succeeds or calls exit(2) to terminate the
- * current process. On success, the file descriptor of the read end of the
- * signal pipe is returned.
+ * \return This function either succeeds or calls exit(3) to terminate the
+ * current process. On success, a signal task structure is returned.
  */